@@ -5,6 +5,8 @@ import colorlog
 import win32com.client
 import platform
 import time
+import base64
+import re
 
 from selenium import webdriver
 from selenium.webdriver.support.ui import WebDriverWait
@@ -15,25 +17,31 @@ from selenium.common.exceptions import TimeoutException
 from selenium.webdriver.common.action_chains import ActionChains
 
 class WebBrowser:
-  def __init__(self, downloadDir = None):
+  def __init__(self, downloadDir = None, captchaDir = None):
     self.logger = logging.getLogger('__main__.' + __name__);
     self.current_path = os.path.abspath(os.getcwd());
 
     # Create Download directory
     self.downloadDir = downloadDir
     if downloadDir == None:
-      downloadDir = os.path.join(current_path, 'ToKkai')
+      self.downloadDir = os.path.join(current_path, 'ToKkai')
     if not os.path.exists(self.downloadDir):
       os.makedirs(self.downloadDir, exist_ok = True)
       self.logger.info("Directory '%s' created" % self.downloadDir)
 
+    self.captchaDir = captchaDir
+    if self.captchaDir != None:
+      if not os.path.exists(self.captchaDir):
+        os.makedirs(self.captchaDir, exist_ok = True)
+        self.logger.info("Directory '%s' created" % self.captchaDir)
+
     # File Chrome
     if (os.path.exists('C:\\Program Files (x86)\\Google\\Chrome\\Application\\chrome.exe')):
       self.chrome_exe = 'C:\\Program Files (x86)\\Google\\Chrome\\Application\\chrome.exe'
     elif (os.path.exists('C:\\Program Files\\Google\\Chrome\\Application\\chrome.exe')):
       self.chrome_exe = 'C:\\Program Files\\Google\\Chrome\\Application\\chrome.exe'
     else:
-      logger.error("Error: không tìm thấy file <folder>\\Google\\Chrome\\Application\\chrome.exe")
+      self.logger.error("Error: không tìm thấy file <folder>\\Google\\Chrome\\Application\\chrome.exe")
       os._exit(1)
 
     # File chromedriver
@@ -58,6 +66,16 @@ class WebBrowser:
                 }
     self.chrome_options.add_experimental_option('prefs', self.preferences)
 
+  def saveImage(self, filename, image_data):
+    try:
+      image_data = re.split(',',image_data)[1]
+      imgdata = base64.b64decode(image_data)
+      with open(filename, 'wb') as f:
+        f.write(imgdata)
+    except OSError:
+      return False
+    return True
+
   def getConfig(self):
     self.logger.info("Current Path: %s" % self.current_path)
     self.logger.info("OS Platform: %s, %s" % (self.os_platform[0],self.os_platform[1]))
@@ -92,13 +110,17 @@ class WebBrowser:
       'Timeout waiting for {}'.format(condition_function.__name__)
     )"""
   def waitForPageLoaded(self):
-    #time.sleep(1)
+    time.sleep(1)
     def page_has_loaded():
-      #self.logger.info("wait for page loaded...")
-      #page_state = self.driver.execute_script('return document.readyState;')
+      self.logger.info("Wait for page loaded...")
+      page_state = self.driver.execute_script('return document.readyState;')
+      return page_state == 'complete'
+    self.waitFor(page_has_loaded)
 
+  def waitForGhiNhan(self):
+    def page_has_loaded():
       status = 'loading'
-      search = self.searchElement("Capcha Empty", 'XPATH', '//input[@formcontrolname="textCaptcha"]', 1, 0, 1)
+      search = self.searchElement("Captcha empty", 'XPATH', '//input[@formcontrolname="maCapcha"]', 1, 0, 1)
       if search:
         search_value = search.get_attribute('value')
         if not search_value or len(search_value) == 0:
@@ -110,7 +132,7 @@ class WebBrowser:
     self.driver.get(url);
     time.sleep(0.5)
     self.driver.refresh()
-    time.sleep(2)
+    self.waitForPageLoaded()
 
   def quitDriver(self):
     self.driver.quit()
@@ -300,38 +322,41 @@ class WebBrowser:
     #so nha
     if dia_chi:
       self.searchSendkeys('So Nha', 'XPATH', '//*[@id="mat-input-11"]', dia_chi, 10, 0,2)
+    #Email
+    #if so_dt_str:
+    #  self.searchSendkeys('So Di Dong', 'XPATH', '//input[@formcontrolname="Email"]', "%s@gmail.com" % (so_dt_str), 10, 0,2)
     #So di dong
     if so_dt_str:
       self.searchSendkeys('So Di Dong', 'XPATH', '//*[@id="mat-input-3"]', so_dt_str, 10, 0,2)
     #Click Chon
     if chon_bhxh_hm:
-      self.searchClick('Cick Chon', 'XPATH', '//*[@id="form-cnt"]/div[9]/div/div/button/span', 30,2,2)
+      self.searchClick('Cick Chon', 'XPATH', '//*[@id="form-cnt"]/div[9]/div/div/button/span', 30,2,1)
       #Chon 079
-      self.searchClick('Chon 079', 'XPATH', '//*[@id="body-dialog"]/tree-root/tree-viewport/div/div/tree-node-collection/div/tree-node[50]/div/tree-node-wrapper/div/div/tree-node-content/span', 30,2,2)
+      self.searchClick('Chon 079', 'XPATH', '//*[@id="body-dialog"]/tree-root/tree-viewport/div/div/tree-node-collection/div/tree-node[50]/div/tree-node-wrapper/div/div/tree-node-content/span', 30,2,1)
       #Chon 07921
-      self.searchClick('Chon 07921', 'XPATH', '//*[@id="body-dialog"]/tree-root/tree-viewport/div/div/tree-node-collection/div/tree-node[50]/div/tree-node-children/div/tree-node-collection/div/tree-node[19]/div/tree-node-wrapper/div/div/tree-node-content/span', 30,2,2)
+      self.searchClick('Chon 07921', 'XPATH', '//*[@id="body-dialog"]/tree-root/tree-viewport/div/div/tree-node-collection/div/tree-node[50]/div/tree-node-children/div/tree-node-collection/div/tree-node[19]/div/tree-node-wrapper/div/div/tree-node-content/span', 30,2,1)
       #Click Chon
-      self.searchClick('Click Chon', 'XPATH', '//*[@id="footer-dialog"]/button[1]', 30,1,2)
+      self.searchClick('Click Chon', 'XPATH', '//*[@id="footer-dialog"]/button[1]', 30,1,1)
 
-    # Go to Capcha
-    self.logger.info("Nhap Capcha")
-    self.moveToElement("Capcha Textbox", 'XPATH', '//input[@formcontrolname="textCaptcha"]', 10, 2,2)
+    # Go to Captcha
+    self.logger.info("Nhap Captcha")
+    self.moveToElement("Captcha Textbox", 'XPATH', '//input[@formcontrolname="maCapcha"]', 10, 2,2)
 
-  def waitCapcha(self, file_cd, file_mt, file_ms, ho_ten, ma_so_str, cmnd_str, tinh_tp_str, huyen_str, xa_str, dia_chi, so_dt_str, chon_bhxh_hm=False):
+  def waitCaptcha(self, file_cd, file_mt, file_ms, ho_ten, ma_so_str, cmnd_str, tinh_tp_str, huyen_str, xa_str, dia_chi, so_dt_str, chon_bhxh_hm=False):
     status = ""
     msg = ""
     is_top = True
     ghi_nhan = False
-    last_capcha = "new"
+    last_Captcha = "new"
     search_value = "new"
-    for i in range(30):
-      search = self.searchElement("[%i] Nhap Capcha" % (i), 'XPATH', '//input[@formcontrolname="textCaptcha"]', 4, 0, 2)
+    for i in range(60):
+      search = self.searchElement("[%i] Nhap Captcha" % (i), 'XPATH', '//input[@formcontrolname="maCapcha"]', 4, 0, 2)
       if search:
         search_value = search.get_attribute('value')
         search_value = search_value.upper()
 
       # Cho nhap
-      if len(search_value) == 0:
+      if len(search_value) == 0 and not ghi_nhan:
         time.sleep(3)
         continue
 
@@ -340,8 +365,8 @@ class WebBrowser:
 
       # Da nhap
       if not '@' in search_value and len(search_value) == 4:
-        if last_capcha != search_value:
-          search = self.searchElement("[%i] Cap nhat Capcha" % (i), 'XPATH', '//input[@formcontrolname="textCaptcha"]', 2, 0, 2)
+        if last_Captcha != search_value:
+          search = self.searchElement("[%i] Cap nhat Captcha" % (i), 'XPATH', '//input[@formcontrolname="maCapcha"]', 2, 0, 2)
           if search:
             search_value = search.get_attribute('value')
             for c in search_value:
@@ -352,15 +377,22 @@ class WebBrowser:
                 search.send_keys(search_value)
                 time.sleep(1)
                 break
+          if self.captchaDir != None:
+            search = self.searchElement("[%i] Captcha image" % (i), 'XPATH', '//img[@alt="captcha"]', 2, 0, 2)
+            if search:
+              self.logger.info("Save Captcha\%s.png" % (search_value))
+              captchaFilepath = os.path.join(self.captchaDir, "%s.png" % (search_value))
+              self.saveImage(captchaFilepath,search.get_attribute('src'))
           # Click 'Ghi Nhan'
           self.searchClick('Ghi Nhan', 'XPATH', '//*[@id="cdk-step-content-0-1"]/app-dang-ky-ca-nhan/form/div/div/div/div[3]/div/button/span', 10, 1, 2)
-          last_capcha = search_value
+          last_Captcha = search_value
           ghi_nhan = True
-          self.waitForPageLoaded()
+          self.searchClick("[%i] Reset Captcha" % (i), 'XPATH', '//button[@aria-label="refresh captcha"]', 2, 0, 2)
+          self.waitForGhiNhan()
 
         # Kiểm tra 'Thông Báo lỗi'
         error_msg = ""
-        search = self.searchElement('Thong Bao', 'XPATH', '//*[@id="toast-container"]/div/div[1]', 2,0,2)
+        search = self.searchElement('Thong Bao', 'XPATH', '//*[@id="toast-container"]/div/div[1]', 2,0,1)
         if search:
           error_msg = search.text
 
@@ -396,24 +428,24 @@ class WebBrowser:
           elif 'Vui lòng điền đầy đủ' in error_msg:
             self.logger.error("Da co loi")
             status = "error"
-            # xóa capcha đã nhập
-            search = self.searchElement('Capcha box-xoa', 'XPATH', '//input[@formcontrolname="textCaptcha"]', 4, 0, 2)
+            # xóa Captcha đã nhập
+            search = self.searchElement('Captcha box-xoa', 'XPATH', '//input[@formcontrolname="maCapcha"]', 4, 0, 2)
             search.clear()
-            # làm mới capcha
-            self.searchClick('Lam moi capcha', 'XPATH', '//*[@id="form-cnt"]/div[11]/div[3]/div/div/div/div[2]/a/img', 4, 0, 2)
+            # làm mới Captcha
+            #self.searchClick('Lam moi Captcha', 'XPATH', '//*[@id="form-cnt"]/div[11]/div[3]/div/div/div/div[2]/a/img', 4, 0, 2)
           #Có lỗi xảy ra
           else:
             status = "error"
-            # xóa capcha đã nhập
-            search = self.searchElement('Capcha box-xoa', 'XPATH', '//input[@formcontrolname="textCaptcha"]', 4, 0, 2)
+            # xóa Captcha đã nhập
+            search = self.searchElement('Captcha box-xoa', 'XPATH', '//input[@formcontrolname="maCapcha"]', 4, 0, 2)
             search.clear()
-            # làm mới capcha
-            self.searchClick('Lam moi capcha', 'XPATH', '//*[@id="form-cnt"]/div[11]/div[3]/div/div/div/div[2]/a/img', 4, 0, 2)
+            # làm mới Captcha
+            # self.searchClick('Lam moi Captcha', 'XPATH', '//*[@id="form-cnt"]/div[11]/div[3]/div/div/div/div[2]/a/img', 4, 0, 2)
       else:
-        if '@' in search_value and last_capcha != search_value:
+        if '@' in search_value and last_Captcha != search_value:
           ghi_nhan = False
-          last_capcha = search_value
-          self.logger.info("Capcha: %s" % search_value.encode('cp1250','replace'))
+          last_Captcha = search_value
+          self.logger.info("Captcha: %s" % search_value.encode('cp1250','replace'))
           if ('XO' in search_value):
             self.logger.warning("Nguoi dung da hoan thanh")
             status = "xong"
